@@ -37,6 +37,7 @@
 #include <gtk/gtkhpaned.h>
 #include <gtk/gtkvpaned.h>
 #include <gtk/gtkimage.h>
+#include <gtk/gtkimagemenuitem.h>
 #include <gtk/gtkitemfactory.h>
 #include <gtk/gtkmain.h>
 #include <gtk/gtkmenubar.h>
@@ -58,6 +59,8 @@
 #define _(x) gettext (x)
 #define N_(x) (x)
 
+static GObjectClass *parent_class;
+
 static char *
 gconf_editor_window_item_factory_translate_func (const char *path, gpointer func_data)
 {
@@ -165,6 +168,8 @@ gconf_editor_window_about_window (gpointer callback_data, guint action, GtkWidge
 				 GTK_WIN_POS_CENTER_ON_PARENT);
 	gtk_window_set_type_hint (GTK_WINDOW (about_window), 
 				  GDK_WINDOW_TYPE_HINT_DIALOG);
+	gtk_window_set_transient_for (GTK_WINDOW (about_window), 
+				      GTK_WINDOW (callback_data));
 
 	vbox = gtk_vbox_new (FALSE, 0);
 	gtk_container_set_border_width (GTK_CONTAINER (vbox), 6);
@@ -195,19 +200,38 @@ gconf_editor_window_about_window (gpointer callback_data, guint action, GtkWidge
 static GtkItemFactoryEntry menu_items[] =
 {
 	{ N_("/_File"),                        NULL,         0,                                   0, "<Branch>" },
+	{ N_("/File/tearoff1"),                NULL,         NULL,                                0, "<Tearoff>" },
 	{ N_("/File/_New window"),             "<control>N", gconf_editor_window_new_window,      0, "<StockItem>", GTK_STOCK_NEW },
 	{ N_("/File/_Close window"),           "<control>W", gconf_editor_window_close_window,    0, "<StockItem>", GTK_STOCK_CLOSE },
 	{ N_("/File/sep1"),                    NULL,         0,                                   0, "<Separator>" },
 	{ N_("/File/_Quit"),                   "<control>Q", gtk_main_quit,                       0, "<StockItem>", GTK_STOCK_QUIT },
 	{ N_("/_Edit"),                        NULL,         0,                                   0, "<Branch>" },
+	{ N_("/Edit/tearoff2"),                NULL,         NULL,                                0, "<Tearoff>" },
 	{ N_("/Edit/_Copy key name"),          NULL,         gconf_editor_window_copy_key_name,   0, "<StockItem>", GTK_STOCK_COPY },
 	{ N_("/_Bookmarks"),                   NULL,         0,                                   0, "<Branch>", },
+	{ N_("/Bookmarks/tearoff3"),           NULL,         NULL,                                0, "<Tearoff>" },
 	{ N_("/Bookmarks/_Add bookmark"),      NULL,         gconf_editor_window_add_bookmark,    0, "<Item>", },
 	{ N_("/Bookmarks/_Edit bookmarks..."), NULL,         gconf_editor_window_edit_bookmarks,  0, "<Item>", },
 	{ N_("/_Help"),                        NULL,         0,                                   0, "<Branch>" },
+	{ N_("/Help/tearoff4"),                NULL,         NULL,                                0, "<Tearoff>" },
 	{ N_("/Help/_About..."),               NULL,         gconf_editor_window_about_window,    0, "<StockItem>", GCONF_STOCK_ABOUT },
 };
 
+const char *tearoff_paths [] = {
+	N_("/File/tearoff1"),
+	N_("/Edit/tearoff2"),
+	N_("/Bookmarks/tearoff3"),
+	N_("/Help/tearoff4")
+};
+
+const char *image_menu_items_paths [] = {
+	N_("/File/New window"),
+	N_("/File/Close window"),
+	N_("/File/Quit"),
+	N_("/Edit/Copy key name"),
+	N_("/Help/About...")
+};
+
 static void
 gconf_editor_window_popup_error_dialog (GtkWindow   *parent,
 					const gchar *message,
@@ -376,12 +400,11 @@ gconf_editor_popup_window_edit_key (gpointer callback_data, guint action, GtkWid
 			    -1);
 
 	if (value && (value->type == GCONF_VALUE_SCHEMA ||
-		      value->type == GCONF_VALUE_PAIR ||
-		      value->type == GCONF_VALUE_LIST)) {
+		      value->type == GCONF_VALUE_PAIR)) {
 		dialog = gtk_message_dialog_new (GTK_WINDOW (window), 0,
 						 GTK_MESSAGE_INFO,
 						 GTK_BUTTONS_OK,
-						 _("Currently lists, pairs and schemas can't "
+						 _("Currently pairs and schemas can't "
 						   "be edited. This will be change in a later "
 						   "version."));
 		gtk_dialog_run (GTK_DIALOG (dialog));
@@ -405,6 +428,15 @@ gconf_editor_popup_window_edit_key (gpointer callback_data, guint action, GtkWid
 	gtk_widget_show (editor);
 }
 
+static void
+gconf_editor_window_list_view_row_activated (GtkTreeView       *tree_view,
+					     GtkTreePath       *path,
+					     GtkTreeViewColumn *column,
+					     GConfEditorWindow *window)
+{
+	gconf_editor_popup_window_edit_key (window, 0, NULL);
+}
+
 static GtkItemFactoryEntry popup_menu_items[] = {
 	{ N_("/_New key..."), NULL, gconf_editor_popup_window_new_key,   0, "<StockItem>", GTK_STOCK_NEW },
 	{ N_("/Edit key..."), NULL, gconf_editor_popup_window_edit_key,  1, "<Item>", NULL }, 
@@ -412,6 +444,11 @@ static GtkItemFactoryEntry popup_menu_items[] = {
 	{ N_("/_Unset key"),  NULL, gconf_editor_popup_window_unset_key, 2, "<StockItem>", GTK_STOCK_DELETE },
 };
 
+const char *image_popup_items_paths [] = {
+	N_("/New key..."),
+	N_("/Unset key"),
+};
+
 static void
 gconf_editor_window_selection_changed (GtkTreeSelection *selection, GConfEditorWindow *window)
 {
@@ -620,10 +657,154 @@ gconf_editor_window_update_list_selection (GtkTreeSelection *selection, GConfEdi
 	g_free (path);
 }
 
+static void
+gconf_editor_window_set_have_tearoffs (GConfEditorWindow *window,
+				       gboolean           have_tearoffs)
+{
+	int i;
+
+	for (i = 0; i < G_N_ELEMENTS (tearoff_paths); i++) {
+		GtkWidget *item;
+
+		item = gtk_item_factory_get_item (window->item_factory, tearoff_paths [i]);
+
+		if (have_tearoffs)
+			gtk_widget_show (item);
+		else
+			gtk_widget_hide (item);
+	}
+}
+
+static void
+gconf_editor_window_have_tearoffs_notify (GConfClient       *client,
+					  guint              cnxn_id,
+					  GConfEntry        *entry,
+					  GConfEditorWindow *window)
+{
+	gboolean have_tearoffs;
+
+	if (entry->value->type != GCONF_VALUE_BOOL)
+		return;
+
+	have_tearoffs = gconf_value_get_bool (entry->value);
+
+	gconf_editor_window_set_have_tearoffs (window, have_tearoffs);
+}
+
+static void
+gconf_editor_window_set_item_has_icon (GtkItemFactory *item_factory,
+				       const char     *path,
+				       gboolean        have_icons)
+{
+	GtkWidget *item;
+	GtkWidget *image;
+
+	item = gtk_item_factory_get_item (item_factory, path);
+
+	image = gtk_image_menu_item_get_image (GTK_IMAGE_MENU_ITEM (item));
+	if (image && !g_object_get_data (G_OBJECT (item), "gconf-editor-icon"))
+		g_object_set_data_full (G_OBJECT (item), "gconf-editor-icon",
+					g_object_ref (image), g_object_unref);
+
+	if (!image)
+		image = g_object_get_data (G_OBJECT (item), "gconf-editor-icon");
+
+	if (!image)
+		return;
+
+	if (have_icons)
+		gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
+	else
+		gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), NULL);
+}
+
+static void
+gconf_editor_window_set_have_icons (GConfEditorWindow *window,
+				    gboolean           have_icons)
+{
+	int i;
+
+	for (i = 0; i < G_N_ELEMENTS (image_menu_items_paths); i++)
+		gconf_editor_window_set_item_has_icon (window->item_factory, image_menu_items_paths [i], have_icons);
+
+	for (i = 0; i < G_N_ELEMENTS (image_popup_items_paths); i++)
+		gconf_editor_window_set_item_has_icon (window->popup_factory, image_popup_items_paths [i], have_icons);
+}
+
+static void
+gconf_editor_window_have_icons_notify (GConfClient       *client,
+				       guint              cnxn_id,
+				       GConfEntry        *entry,
+				       GConfEditorWindow *window)
+{
+	gboolean have_icons;
+
+	if (entry->value->type != GCONF_VALUE_BOOL)
+		return;
+
+	have_icons = gconf_value_get_bool (entry->value);
+
+	gconf_editor_window_set_have_icons (window, have_icons);
+}
+
+static void
+gconf_editor_window_setup_ui_prefs_handler (GConfEditorWindow *window)
+{
+	GConfClient *client;
+
+	client = gconf_client_get_default ();
+
+	gconf_editor_window_set_have_tearoffs (
+		window, gconf_client_get_bool (client, "/desktop/gnome/interface/menus_have_tearoff", NULL));
+
+	window->tearoffs_notify_id = gconf_client_notify_add (
+						client,
+						"/desktop/gnome/interface/menus_have_tearoff",
+						(GConfClientNotifyFunc) gconf_editor_window_have_tearoffs_notify,
+						window, NULL, NULL);
+
+	gconf_editor_window_set_have_icons (
+		window, gconf_client_get_bool (client, "/desktop/gnome/interface/menus_have_icons", NULL));
+
+	window->icons_notify_id = gconf_client_notify_add (
+						client,
+						"/desktop/gnome/interface/menus_have_icons",
+						(GConfClientNotifyFunc) gconf_editor_window_have_icons_notify,
+						window, NULL, NULL);
+
+	g_object_unref (client);
+}
+
+static void
+gconf_editor_window_finalize (GObject *object)
+{
+        GConfEditorWindow *window = (GConfEditorWindow *) object;
+	GConfClient       *client;
+
+	if (window->item_factory)
+		g_object_unref (window->item_factory);
+
+	client = gconf_client_get_default ();
+
+	if (window->tearoffs_notify_id)
+		gconf_client_notify_remove (client, window->tearoffs_notify_id);
+
+	if (window->icons_notify_id)
+		gconf_client_notify_remove (client, window->icons_notify_id);
+
+	g_object_unref (client);
+
+        parent_class->finalize (object);
+}
 
 static void
 gconf_editor_window_class_init (GConfEditorWindowClass *klass)
 {
+	GObjectClass *object_class = (GObjectClass *) klass;
+
+	object_class->finalize = gconf_editor_window_finalize;
+
+	parent_class = g_type_class_peek_parent (klass);
 }
 
 static void
@@ -660,14 +841,10 @@ gconf_editor_window_init (GConfEditorWindow *window)
 	
 	item_factory = gtk_item_factory_new (GTK_TYPE_MENU_BAR, "<main>", accel_group);
 	gtk_item_factory_set_translate_func (item_factory, gconf_editor_window_item_factory_translate_func, NULL, NULL);
-	
+
 	/* Set up item factory to go away with the window */
-	g_object_ref (item_factory);
+	window->item_factory = g_object_ref (item_factory);
 	gtk_object_sink (GTK_OBJECT (item_factory));
-	g_object_set_data_full (G_OBJECT (window),
-				"<main>",
-				item_factory,
-				(GDestroyNotify) g_object_unref);
 	
 	/* Create menu items */
 	gtk_item_factory_create_items (item_factory, G_N_ELEMENTS (menu_items),
@@ -735,6 +912,8 @@ gconf_editor_window_init (GConfEditorWindow *window)
 
 	g_signal_connect (window->list_view, "popup_menu",
 			  G_CALLBACK (gconf_editor_window_list_view_popup_menu), window);
+	g_signal_connect (window->list_view, "row_activated",
+			  G_CALLBACK (gconf_editor_window_list_view_row_activated), window);
 
 	g_signal_connect (gtk_tree_view_get_selection (GTK_TREE_VIEW (window->list_view)), "changed",
 			  G_CALLBACK (gconf_editor_window_update_list_selection), window);
@@ -845,11 +1024,11 @@ gconf_editor_window_init (GConfEditorWindow *window)
 	window->long_desc_buffer = gtk_text_buffer_new (NULL);
 	gtk_text_buffer_set_text (window->long_desc_buffer, _("(None)"), -1);	
 	text_view = gtk_text_view_new_with_buffer (window->long_desc_buffer);
+	gtk_widget_modify_base (text_view, GTK_STATE_NORMAL,
+				&GTK_WIDGET (window)->style->bg[GTK_STATE_NORMAL]);
 	gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (text_view), GTK_WRAP_WORD);
 	gtk_text_view_set_editable (GTK_TEXT_VIEW (text_view), FALSE);
 	scrolled_window = gtk_scrolled_window_new (NULL, NULL);
-	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolled_window),
-					     GTK_SHADOW_IN);
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
 					GTK_POLICY_AUTOMATIC,
 					GTK_POLICY_AUTOMATIC);
@@ -861,13 +1040,14 @@ gconf_editor_window_init (GConfEditorWindow *window)
 
 	gtk_widget_show_all (vbox);
 
-
 	/* Set window icon */
 	icon = gdk_pixbuf_new_from_file (IMAGEDIR"/gconf-editor.png", NULL);
 	if (icon != NULL) {
 		gtk_window_set_icon (GTK_WINDOW (window), icon);
 		g_object_unref (icon);
 	}
+
+	gconf_editor_window_setup_ui_prefs_handler (window);
 }
 
 GType
