@@ -48,6 +48,7 @@ struct _GConfKeyEditor {
 	GtkWidget *bool_widget;
 	GtkWidget *int_widget;
 	GtkWidget *string_widget;
+	GtkWidget *float_widget;
 	
 	GtkWidget *path_label;
 	GtkWidget *path_box;
@@ -56,6 +57,8 @@ struct _GConfKeyEditor {
 	/* List editing stuff */
 	GtkWidget *list_widget;
 	GtkListStore *list_model;
+	GtkWidget *list_type_menu;
+	GtkWidget *remove_button, *go_up_button, *go_down_button;
 
 };
 
