@@ -81,8 +81,10 @@ gconf_client_get_schema_for_key (GConfClient *client, const char *key)
 	schema_name = gconf_entry_get_schema_name (entry);
 	
 	if (schema_name == NULL) {
+#if 0
 		/* Clear the cache and try again */
-//		gconf_client_clear_cache (client);
+		gconf_client_clear_cache (client);
+#endif
 		entry = gconf_client_get_entry (client, key, NULL, TRUE, NULL);
 		schema_name = gconf_entry_get_schema_name (entry);
 	}
